==> nivo-0.15.0.tgz: package/es/lib/scale.js <==
import _ from 'lodash-es';
import PropTypes from 'prop-types';
import { scaleLinear, scaleBand, scalePoint } from 'd3';

var SCALE_LINEAR = 'linear';
var SCALE_BAND = 'band';
var SCALE_POINT = 'point';

export var scaleTypes = [SCALE_LINEAR, SCALE_BAND, SCALE_POINT];

export var scalePropTypes = {
    type: PropTypes.oneOf(scaleTypes).isRequired,
    axis: PropTypes.oneOf(['x', 'y']).isRequired,
    dataKey: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.func]),
    aggregate: PropTypes.array,
    maxOf: PropTypes.array,
    padding: PropTypes.number
};

export var scalesPropType = function scalesPropType(props, propName, componentName) {
    var scales = props[propName];

    _.forOwn(scales, function (scale, id) {
        PropTypes.checkPropTypes(scalePropTypes, scale, '`' + propName + '.' + id + '`', componentName);
    });
};

export var createScale = function createScale(_ref, data, width, height) {
    var type = _ref.type,
        axis = _ref.axis,
        key = _ref.key,
        aggregate = _ref.aggregate,
        maxOf = _ref.maxOf,
        padding = _ref.padding;

    var mapper = void 0;
    if (Array.isArray(aggregate)) {
        mapper = function mapper(d) {
            return _.sum(aggregate.map(function (k) {
                return d[k];
            }));
        };
    } else if (Array.isArray(maxOf)) {
        mapper = function mapper(d) {
            return _.max(maxOf.map(function (k) {
                return d[k];
            }));
        };
    } else if (_.isFunction(key)) {
        mapper = key;
    } else {
        mapper = function mapper(d) {
            return d[key];
        };
    }

    var range = [axis === 'y' ? height : 0, axis === 'x' ? width : 0];

    var scale = void 0;
    switch (type) {
        case SCALE_LINEAR:
            scale = scaleLinear().rangeRound(range).domain([0, _.max(data.map(mapper))]);
            break;

        case SCALE_BAND:
            scale = scaleBand().rangeRound(range).domain(data.map(mapper));

            if (padding !== undefined) {
                scale.padding(padding);
            }
            break;

        case SCALE_POINT:
            scale = scalePoint().range(range).domain(data.map(mapper));
            break;
    }

    return scale;
};

export var scalesFromObject = function scalesFromObject(object, width, height, data) {
    return _.mapValues(object, function (scale, id) {
        return createScale(Object.assign({}, scale, { id: id }), data, width, height);
    });
};

==> nivo-0.21.0.tgz: package/es/lib/scale.js <==
import _mapValues from 'lodash/mapValues';
import _isFunction from 'lodash/isFunction';
import _max from 'lodash/max';
import _sum from 'lodash/sum';
import _forOwn from 'lodash/forOwn';

import PropTypes from 'prop-types';
import { scaleLinear, scaleBand, scalePoint } from 'd3';

var SCALE_LINEAR = 'linear';
var SCALE_BAND = 'band';
var SCALE_POINT = 'point';

export var scaleTypes = [SCALE_LINEAR, SCALE_BAND, SCALE_POINT];

export var scalePropTypes = {
    type: PropTypes.oneOf(scaleTypes).isRequired,
    axis: PropTypes.oneOf(['x', 'y']).isRequired,
    dataKey: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.func]),
    aggregate: PropTypes.array,
    maxOf: PropTypes.array,
    padding: PropTypes.number
};

export var scalesPropType = function scalesPropType(props, propName, componentName) {
    var scales = props[propName];

    _forOwn(scales, function (scale, id) {
        PropTypes.checkPropTypes(scalePropTypes, scale, '`' + propName + '.' + id + '`', componentName);
    });
};

export var createScale = function createScale(_ref, data, width, height) {
    var type = _ref.type,
        axis = _ref.axis,
        key = _ref.key,
        aggregate = _ref.aggregate,
        maxOf = _ref.maxOf,
        padding = _ref.padding;

    var mapper = void 0;
    if (Array.isArray(aggregate)) {
        mapper = function mapper(d) {
            return _sum(aggregate.map(function (k) {
                return d[k];
            }));
        };
    } else if (Array.isArray(maxOf)) {
        mapper = function mapper(d) {
            return _max(maxOf.map(function (k) {
                return d[k];
            }));
        };
    } else if (_isFunction(key)) {
        mapper = key;
    } else {
        mapper = function mapper(d) {
            return d[key];
        };
    }

    var range = [axis === 'y' ? height : 0, axis === 'x' ? width : 0];

    var scale = void 0;
    switch (type) {
        case SCALE_LINEAR:
            scale = scaleLinear().rangeRound(range).domain([0, _max(data.map(mapper))]);
            break;

        case SCALE_BAND:
            scale = scaleBand().rangeRound(range).domain(data.map(mapper));

            if (padding !== undefined) {
                scale.padding(padding);
            }
            break;

        case SCALE_POINT:
            scale = scalePoint().range(range).domain(data.map(mapper));
            break;
    }

    return scale;
};

export var scalesFromObject = function scalesFromObject(object, width, height, data) {
    return _mapValues(object, function (scale, id) {
        return createScale(Object.assign({}, scale, { id: id }), data, width, height);
    });
};
